#ifndef __MV_COM_FLASH_H__
#define __MV_COM_FLASH_H__

#include "com_define.h"
#include "com_event_struct.h"

#define DRIVER_LENGTH                      1024*16

#ifdef   SUPPORT_CONFIG_FILE
#define   AUTOLOAD_FLASH_OFFSET			0x38
#define   AUTOLOAD_FLASH_LENGTH			0x108
#define	RAID_BIOS_ROM_SIZE				(64*1024L)
#define   RELOCATE_ROM_SIZE					0x1000
#define   ROM_IMAGE_4K_CHECKSUM 			0x36  //byte
#endif


#ifndef _OS_BIOS
#pragma pack(8)
#endif /* _OS_BIOS */

typedef struct _Flash_DriverData
{
	MV_U16            Size;
	MV_U8             PageNumber;
	MV_BOOLEAN        isLastPage;
	MV_U16            Reserved[2];
	MV_U8             Data[DRIVER_LENGTH];
}
Flash_DriveData, *PFlash_DriveData;

#if defined(SAVE_EVENTS_TO_FLASH)
#define MAX_FLASH_EVENT_LOG_ENTRY	((128 * 1024 * 2 / 128) - 2) /*A block is 128K in Flash. Event Log will use 2 blocks. two will be to use the event log header(first and last, they will in different block)*/
#define EVENT_LOG_VERSION 0x000010C

/*The size of flash_event_log_entry equal to flash_event_log_head*/
struct flash_event_log_entry
{
#if defined(ROC_V2_EVENT_SUPPORT)
 	DriverEvent_V2 event;
#else
	DriverEvent event;
#endif
	MV_U32 crc;

	MV_U8    valid;
	MV_U8    reserved0[7];

#if defined(ROC_V2_EVENT_SUPPORT)
	MV_U8  	reserved1[52]; 
#else
	MV_U8  	reserved1[84]; 
#endif
       /*please to keep total is 128 bytes.*/
};

struct flash_event_log_head
{
	MV_U8	signature[4];
	MV_U32	next_tbl_off;
	MV_U16	version;
	MV_U16  	nr_max_entries;
	MV_U32  	crc;
	MV_U8  	reserved[112];
       /*please to keep total is 128 bytes.*/
};
#endif /*SAVE_EVENTS_TO_FLASH*/

#if defined(ADDING_FLASH_LAYOUT_DESC)
struct flash_layout_item
{
        MV_U16    module_type;
        MV_U16    id;

        MV_U32    reserved0[1];
        MV_U32    start_address;
        MV_U32    size;
};


struct flash_layout_head
{
        MV_U8      signature[8];

        MV_U32    next_tbl_off;

        MV_U16    version;
        MV_U16    reserved0[1];

        MV_U32    length;

        MV_U32    crc;

        MV_U32    reserved1[2];

        struct flash_layout_item item[1];
};
#define FLASH_LAYOUT_SIGN "MRVL_MAP"
#define FLASH_LAYOUT_SIGN_LENGTH MV_StrLen(FLASH_LAYOUT_SIGN)
//#define FLASH_LAYOUT_OFFSET	return_image_place_on_flash(FLASH_IMAGE_LAYOUTDESC , 0, FLASH_IMAGE_RETURN_ADDRESS)
#define FLASH_LAYOUT_START_ADDR	0x00000000
#define FLASH_LAYOUT_OFFSET		0x00100000

/*The u-boot Start address to be decide by Flash Size(Please check Flash Library).
   To got the SingleBlock and FlashSize by the Flash Library(if have any concern, please check Flash Library).*/

/* Note : all images in description layout must be 32k multiplied, if using the image update or backup feature*/
/*Note 1: The Layout String had been removed. Now it will be put by MVFlash or Windows Loader.*/
#if 0
#define FLASH_LAYOUT_DESC 	"Loki(NOR FLASH Layout) 1 BLOCK = 128K\
			\n{\"BIOS\",0,[0x00000000],[32k],[1 Block]},\
			\n{\"HBA.txt\",0,[0x000020000],[32k],[1 Block]},\
			\n{\"HBA_Info\",0,[0x00040000],[32k],[1 Block]},\
			\n{\"PD_Page\",0,[0x00060000],[128k],[1 Block]},\
			\n{\"Event_Log\",0,[0x00080000],[256k],[2 Blocks]},\
			\n{\"Desc.txt\",0,[0x000c0000],[32k],[1 Block]},\
			\n{\"HBA_Info\",1,[0x000e0000],[32k],[1 Block]},\
			\n{\"Firmware\",0,[0x00100000],[1024k],[4 Blocks]},\
			\n{\"Firmware\",1,[0x00200000],[1024k],[4 Blocks]},\
			\n{\"u-boot\",0,[0x00ff0000],[192k],[2 Block]},\
			\n{\"SingleBlock\",0,[0xffffffff],[128k],[1 Block]},\
			\n{\"FlashSize\",0,[0xffffffff],[16M],[128 Blocks]};"
#endif

enum {
    FLASH_IMAGE_CONFIG = 0x000000000, /*The config had merged the HBA.txt and Layout Description */
    FLASH_IMAGE_HBAINFO,
    FLASH_IMAGE_FMLOADER,
    FLASH_IMAGE_FIRMWARE,
    FLASH_IMAGE_BIOS,
    FLASH_IMAGE_EVENTLOG,
    FLASH_IMAGE_PDINFOPAGE,
    FLASH_IMAGE_SINGLEBLOCK,
    FLASH_IMAGE_FLASHSIZE,
    FLASH_IMAGE_MAX
};

#define FLASH_IMAGE_RETURN_ADDRESS 0
#define FLASH_IMAGE_RETURN_SIZE    1

#ifdef SCSI_RW_BUFFER_CMD
#define GENERATION_SIZE	8

/* flash operation error type define
0: FLASH_NONE_ERR
1: FLASH_WRITE_ERR
2: FLASH_READ_ERR
3: GENERATION_ERR
4: FATAL_GENERATION_ERR
5: IMAGE_DATA_ERR
6: IMAGE_HEADER_ERR
7: IMAGE_HEADER_AND_DATA_ERR
*/
#define    FLASH_NONE_ERR					0
#define    FLASH_WRITE_ERR					1
#define    FLASH_READ_ERR					2
#define    GENERATION_ERR					3
#define    FATAL_GENERATION_ERR			4 // dual image's genreation all crashed
#define    IMAGE_DATA_ERR					5 // single image's generation crashed
#define    IMAGE_HEADER_ERR					6
#define    IMAGE_HEADER_AND_DATA_ERR		7

#define FLASH_DPRINT	 FM_PRINT

#ifdef FLASH_IMAGE_HEADER_CHECK
static MV_U32	global_header_check = 1;
#else
static MV_U32	global_header_check = 0;
#endif
void flash_err_event_generate(MV_VOID * p_core, MV_U8 err_type);
#endif

MV_U32 return_image_place_on_flash(MV_U32 component_type, MV_U16 component_number, MV_U16 component_return_type);

#endif

#ifndef _OS_BIOS
#pragma pack()
#endif /* _OS_BIOS */

#endif /* __MV_COM_FLASH_H__ */
